//+------------------------------------------------------------------+
//|                                                  Correlation.mq5 |
//|                                               Copyright 2012, iC |
//|                                         http://www.icreator.biz/ |
//+------------------------------------------------------------------+
#property copyright "Copyright 2012, iC"
#property link      "http://www.icreator.biz/"
#property version   "1.00"
//--- indicator settings
#property indicator_separate_window
#property indicator_minimum -1
#property indicator_maximum 1
#property indicator_buffers 5
#property indicator_plots   2
#property indicator_type1   DRAW_COLOR_LINE
#property indicator_width1  2
#property indicator_type2   DRAW_COLOR_HISTOGRAM
#property indicator_style2  STYLE_DOT
#property indicator_width2  1
//--- defines
#define MAX_COL 64
//--- structures
struct CRGB
  {
   int               r;
   int               g;
   int               b;
  };
//--- input parametres
input string _SecondSymbol="EURUSD";                   // Symbol
input int _SettPeriod=20;                              // Period
input ENUM_APPLIED_PRICE _AppliedPrice=PRICE_WEIGHTED; // Price
input color _Color1=clrLightGray;                      // Min correlation
input color _Color2=clrLime;                           // Max correlation
//--- indicator buffers
double buf[],buf2[];
double arr1[],arr2[];
double colors1[],colors2[];
//--- MA handles
int h1,h2;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
   SetIndexBuffer(0,buf);
   SetIndexBuffer(1,colors1,INDICATOR_COLOR_INDEX);
   SetIndexBuffer(2,buf2);
   SetIndexBuffer(3,colors2,INDICATOR_COLOR_INDEX);
   SetIndexBuffer(4,arr1,INDICATOR_CALCULATIONS);
   SetIndexBuffer(5,arr2,INDICATOR_CALCULATIONS);
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,0);
   PlotIndexSetDouble(1,PLOT_EMPTY_VALUE,0);
   PlotIndexSetInteger(0,PLOT_SHOW_DATA,0);
   IndicatorSetInteger(INDICATOR_DIGITS,3);
   IndicatorSetString(INDICATOR_SHORTNAME,_Symbol+"/"+_SecondSymbol+", "+IntegerToString(_SettPeriod)+", "+EnumToString(_AppliedPrice)+",");
   setPlotColor(0,_Color1,_Color2);
   setPlotColor(1,_Color1,_Color2);
   h1=iMA(_Symbol,0,1,0,MODE_SMA,_AppliedPrice);
   h2=iMA(_SecondSymbol,0,1,0,MODE_SMA,_AppliedPrice);
   return 0;
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const int begin,
                const double &price[])
  {
   int i,j,c,limit,bars1,bars2,minBars,toCopy;
   double averX1=0,averX2=0,v1=0,v2=0,v3=0;
   datetime t1[1],t2[1];
//--- synchronization symbols
   if((CopyTime(_Symbol,0,0,1,t1)<1 || 
      CopyTime(_SecondSymbol,0,0,1,t2)<1) || 
      t1[0]!=t2[0])
     {
      Print("Synchronization failed!");
      return 0;
     }
//--- check for data
   bars1=rates_total;
   bars2=Bars(_SecondSymbol,0);
   if(bars1>bars2)
     {
      minBars=bars2;
      bars2=bars1-bars2;
      bars1=0;
     }
   else
     {
      minBars=bars1;
      bars1=bars2-bars1;
      bars2=0;
     }
   if(minBars<_SettPeriod)
     {
      Print("Bars is not enough to calculate!");
      return 0;
     }
   if(BarsCalculated(h1)<minBars)
     {
      Print("Not all data of MA1 is calculated. Error ",GetLastError());
      return 0;
     }
   if(BarsCalculated(h2)<minBars)
     {
      Print("Not all data of MA2 is calculated. Error ",GetLastError());
      return 0;
     }
//--- we can copy not all data
   if(prev_calculated>rates_total || 
      prev_calculated<=0)
      toCopy=minBars;
   else
      toCopy=rates_total-prev_calculated+1;
   if(CopyBuffer(h1,0,0,minBars,arr1)<minBars || 
      CopyBuffer(h2,0,0,minBars,arr2)<minBars)
     {
      Print("Getting MA buffer failed. Error ",GetLastError());
      return 0;
     }
   if(prev_calculated>rates_total || 
      prev_calculated<=0)
      limit=bars1+bars2+_SettPeriod-1;
   else
      limit=prev_calculated-1;
//---        
   for(i=limit;i<rates_total;i++)
     {
      averX1=0; averX2=0;
      for(j=0;j<_SettPeriod;j++)
        {
         averX1+=arr1[i-j-bars1]/_SettPeriod;
         averX2+=arr2[i-j-bars2]/_SettPeriod;
        }
      v1=0; v2=0; v3=0;
      for(j=0;j<_SettPeriod;j++)
        {
         v1+=(arr1[i-j-bars1]-averX1)*(arr2[i-j-bars2]-averX2);
         v2+=pow((arr1[i-j-bars1]-averX1),2);
         v3+=pow((arr2[i-j-bars2]-averX2),2);
        }
      buf[i]=v1/sqrt(v2*v3);
      buf2[i]=buf[i];
      c=getPlotColor(buf[i]);
      colors1[i]=c;
      colors2[i]=c;
     }
   return rates_total;
  }
//+------------------------------------------------------------------+
//| setPlotColor                                                     |
//+------------------------------------------------------------------+
void setPlotColor(int plot,color col1,color col2)
  {
   int i;
   CRGB c1,c2;
   double dr,dg,db;
   string s;
//---    
   PlotIndexSetInteger(plot,PLOT_COLOR_INDEXES,MAX_COL);
   ColorToRGB(col1,c1);
   ColorToRGB(col2,c2);
   dr=(double)(c2.r-c1.r)/MAX_COL;
   dg=(double)(c2.g-c1.g)/MAX_COL;
   db=(double)(c2.b-c1.b)/MAX_COL;
   for(i=0;i<MAX_COL;i++)
     {
      s=StringFormat("%i,%i,%i",
                     c1.r+(int)NormalizeDouble(dr*(i+1),0),
                     c1.g+(int)NormalizeDouble(dg*(i+1),0),
                     c1.b+(int)NormalizeDouble(db*(i+1),0));
      PlotIndexSetInteger(plot,PLOT_LINE_COLOR,i,StringToColor(s));
     }
  }
//+------------------------------------------------------------------+
//| getPlotColor                                                     |
//+------------------------------------------------------------------+
int getPlotColor(double current)
  {
   return((int)NormalizeDouble((MAX_COL-1)*fabs(current),0));
  }
//+------------------------------------------------------------------+
//| ColorToRGB                                                       |
//+------------------------------------------------------------------+
void ColorToRGB(color col,CRGB &res)
  {
   string s,s2;
   int n;
//---
   s=ColorToString(col);
   n=StringFind(s,",");
   s2=StringSubstr(s,0,n);
   res.r=(int)StringToInteger(s2);
   s=StringSubstr(s,n+1);
   n=StringFind(s,",");
   s2=StringSubstr(s,0,n);
   res.g=(int)StringToInteger(s2);
   s=StringSubstr(s,n+1);
   s2=StringSubstr(s,0);
   res.b=(int)StringToInteger(s2);
  }
//+------------------------------------------------------------------+
